    
# global imports
from OFS.Folder import Folder
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import os.path
import Globals
import AccessControl
import permissions


# package imports

import interfaces
import repo

### global constants
# status and resolutions
STATUS_OPEN = "OPEN"
STATUS_CLOSED = "CLOSED"
ALL_STATUS = [ "NEW",
               "ANALISYS",
               "CLARIFY",
               "RE-SENT",
               "FIXING",
               "TESTING",
               "FIXED",
               "APROVED",
               "DISCARDED",
               "NON-BUG",
               "POSTPONED", ]

STATUS_TRANSLATIONS = {}
STATUS_TRANSLATIONS["en"] = [ "New",
               "Analisys",
               "Clarify",
               "Re-Sent",
               "Fixing",
               "Testing",
               "Fixed",
               "Aproved",
               "Discarded",
               "Non-Bug",
               "Postponed", ]

STATUS_TRANSLATIONS["pt_BR"] = [ "Novo",
                                 "Anlise",
                                 "Esclarecer",
                                 "Re-Enviado",
                                 "Reparo",
                                 "Teste",
                                 "Consertado",
                                 "Aprovado",
                                 "Descartado",
                                 "No-Bug",
                                 "Adiado", ]


status_transitions = { "NEW"      :["NEW"      ,"ANALISYS"],
                       "ANALISYS" :["ANALISYS" ,"FIXING","CLARIFY",
                                    "DISCARDED","NON-BUG","POSTPONED"],
                       "CLARIFY"  :["CLARIFY"  ,"RE-SENT"],
                       "RE-SENT"  :["RE-SENT"  ,"ANALISYS"],
                       "FIXING"   :["FIXING"   ,"FIXED","TESTING"],
                       "TESTING"  :["TESTING"  ,"FIXING","FIXED"],
                       "FIXED"    :["FIXED"    ,"APROVED","RE-SENT"],
                       "APROVED"  :["APROVED"  ],
                       "DISCARDED":["DISCARDED"],
                       "NON-BUG"  :["NON-BUG"  ],
                       "POSTPONED":["POSTPONED", "RE-SENT"] }



ALL_STATUS_RESULTS = {}
def setup_ALL_STATUS_RESULTS():
    # assemble ALL_STATUS_RESULTS,
    # deciding the resolution of each status
    iopen = ALL_STATUS.index("APROVED")
    for i in range(iopen):
        ALL_STATUS_RESULTS[ALL_STATUS[i]] = STATUS_OPEN
    for i in range(iopen, len(ALL_STATUS)):
        ALL_STATUS_RESULTS[ALL_STATUS[i]] = STATUS_CLOSED

setup_ALL_STATUS_RESULTS()

ALL_PRIORITIES = map(str, range(1,6))

ALL_SEVERITIES = ['CRITICAL',
                  'UNAVOIDABLE',
                  'AVOIDABLE',
                  'COSMETIC',
                  'IMPROVEMENT',
                  ]
    
_wwwDir = os.path.join(os.path.dirname(__file__), 'www')

class Bug(Folder):
    # security object to make calls ans objects secure
    security = AccessControl.ClassSecurityInfo()
    security.declareObjectProtected(AccessControl.Permissions.access_contents_information)
    security.setPermissionDefault(permissions.ADD_BUG, ('Manager', 'Customer'))
    security.setPermissionDefault(permissions.EDIT_BUG, ('Manager',))
    
    __implements__ = interfaces.IBug

    ### Zope standard attributes
    meta_type = "DDBug Bug"
    
    properties = {'reported_by': {'type':'string', 'mode':'w', 'editable_by':''},
                  'summary': {'type':'text', 'mode':'w', 'editable_by':'CP'},
                  'status': {'type':'selection', 'mode':'w','select_variable':'optionsForStatus', 'editable_by':'CP'},
                  'priority': {'type':'selection', 'mode':'w','select_variable':'optionsForPriority', 'editable_by':'C'},
                  'severity': {'type':'selection', 'mode':'w','select_variable':'optionsForSeverity', 'editable_by':'P'},
                  'error_message': {'type':'text', 'mode':'w', 'editable_by':'CP'},
                  'url': {'type':'string', 'mode':'w', 'editable_by':'CP'},
                  'browser': {'type':'string', 'mode':'w', 'editable_by':'CP'},
                  'steps_to_reproduce': {'type':'text', 'mode':'w', 'editable_by':'CP'},
                  'comments': {'type':'text', 'mode':'w', 'editable_by':'CP'},
                  'assigned_to': {'type':'string', 'mode':'w', 'editable_by':''},
                  'workaround': {'type':'text', 'mode':'w', 'editable_by':'P'},
                  'estimated_time_to_fix': {'type':'int', 'mode':'w', 'editable_by':'P'},
                  'actual_time_to_fix': {'type':'int', 'mode':'w', 'editable_by':'P'},
                  'log': {'type':'lines', 'mode':'w', 'editable_by':''},
    }

    # food for PropertyManager 
    _properties = []
    for key, value in properties.items():
        property = {}
        property['id'] = key
        dict = value.copy()
        del dict['editable_by'] # remove field not used by PropertyManager
        property.update(dict)
        _properties.append(property)
    
    _properties = tuple(_properties)

    # and now, the defaults of the above properties
    reported_by = ''
    summary = ''
    status = '' # it will be set in __init__
    priority = '5'
    severity = 'IMPROVEMENT'
    error_message = ''
    url = ''
    browser = ''
    steps_to_reproduce = ''
    comments = ''
    assigned_to = ''
    workaround = ''
    estimated_time_to_fix = 0
    actual_time_to_fix = 0
    log = None

    ALL_STATUS = ALL_STATUS    

    security.declareProtected(AccessControl.Permissions.access_contents_information, 'title')
    def title(self):
        return self.summary[:200]

    def __init__(self, id, priority=priority):
        self.id = id
        self.priority = priority
        self.status = self.initialStatus()
        self.log = []

    security.declarePrivate('_translateStatus')
    def _translateStatus(self, status):
        """Bug status translation method
        Returns the name of a status according to the repository language.
        """
        i = ALL_STATUS.index(status)
        return STATUS_TRANSLATIONS[self.getRepository().language][i]

    security.declareProtected(AccessControl.Permissions.view, 'initialStatus')
    def initialStatus(self):
        return ALL_STATUS[0]

    security.declareProtected(AccessControl.Permissions.view, 'optionsForStatus')
    def optionsForStatus(self):
        return ALL_STATUS

    security.declareProtected(AccessControl.Permissions.view, 'optionsForPriority')
    def optionsForPriority(self):
        return ALL_PRIORITIES

    security.declareProtected(AccessControl.Permissions.view, 'optionsForSeverity')
    def optionsForSeverity(self):
        return ALL_SEVERITIES

    security.declareProtected(AccessControl.Permissions.view, 'getStatusTranslated')
    def getStatusTranslated(self, status=None):
        if not status:
            status = self.status
        return self._translateStatus(status)

    security.declareProtected(AccessControl.Permissions.view, 'detectBrowser')
    def detectBrowser(self):
        """ The world's simplest browser detection method.
            Some day we will have a consolidated list of browser types,
            and this method will match the HTTP_USER_AGENT with one
            of them.
        """    
        return self.REQUEST['HTTP_USER_AGENT']

    security.declareProtected(permissions.EDIT_BUG, 'manage_editBug')
    def manage_editBug(self, REQUEST=None, **kw):
        """Bug editing method
        Changes the bug's properties
        """
        if REQUEST is None:
            props={}
        else: props=REQUEST
        for name, value in props.items():
            if self.hasProperty(name) or name == 'attachment':
                kw[name]=value
        #log
        transition = ''
        if kw.has_key('status') and self.status != kw['status']:
            transition = "Status alterated from %s to %s by %s" % (
                         self.getStatusTranslated(),
                         self.getStatusTranslated(kw['status']),
                         self.getAuthenticatedUser())
        elif not self.log:
            transition = "Bug created by %s" % self.getAuthenticatedUser()
        if transition:
            log = self.log
            log.append(transition)
            kw['log'] = log
        #comments
        if kw.has_key('comments') and kw['comments'] != '':
            if self.comments:
                kw['comments'] = self.comments + '\n' + kw['comments']
            else:
                kw['comments'] = kw['comments']
        else:
            kw['comments'] = self.comments

        #assigned_to
        user = AccessControl.getSecurityManager().getUser()
        roles = user.getRolesInContext(self)
        if 'Manager' in roles:
            kw['assigned_to'] = str(user)
        elif kw.has_key('assigned_to'):
            del kw['assigned_to']
        self.manage_changeProperties(**kw)
        if kw.has_key('attachment') and kw['attachment']:
            self.manage_addFile(id='',title='', file=kw['attachment'])
        if REQUEST:
            self.REQUEST.RESPONSE.redirect(self.aq_parent.absolute_url())

    security.declareProtected(AccessControl.Permissions.view, 'getTransitionOptions')
    def getTransitionOptions(self):
        """getTransitionOptions() -> list of dicts
        Returns all the next possible status for a bug, in a format suitable
        for using by the options of a select widget in a form:
        [ { 'value':'STATUSVALUE', 'content':'Status Name', 'selected':<0|1> },
          ...
        ]
        """
        statusMap = []
        for status in status_transitions.get(self.status, ['NEW']):
            statusItem = {'value':status,
                          'content':self._translateStatus(status),
                          'selected': status == self.status}
            statusMap.append(statusItem)
        # statusMap will be [] if our status is invalid.
        # schema.pt is expecting this and won't render a status field.
        # this is expected to happen for when schema is rendered in
        # the MasterBug, and Repository.newBug() deals with that
        # by setting the status of the new bug to .initialStatus()
        return statusMap

    security.declareProtected(AccessControl.Permissions.view, 'open')
    def open(self):
        return ALL_STATUS_RESULTS[self.status] == STATUS_OPEN
        
    security.declareProtected(AccessControl.Permissions.view, 'editableProperty')
    def editableProperty(self, property_name):
        '''template helper '''
        '''return true if property should be editable by current user'''
        user = AccessControl.getSecurityManager().getUser()
        roles = user.getRolesInContext(self)
        # for now, all programmer actors are managers
        actorCodes = []
        if 'Manager' in roles:
            actorCodes.append('P')
        if 'Customer' in roles:
            actorCodes.append('C')
        if not actorCodes:
            return 0
        for actorCode in actorCodes:
            if actorCode in self.properties[property_name]['editable_by']:
                return 1
        return 0
        
    security.declareProtected(AccessControl.Permissions.view, 'visibleProperty')
    def visibleProperty(self, property_name):
        '''template helper '''
        '''return true if property should be displayed to current user'''
        if getattr(self, property_name):
            return 1
        elif self.editableProperty(property_name):
            return 1
        else:
            return 0

    security.declareProtected(AccessControl.Permissions.view, 'listAttachments')
    def listAttachments(self):
        '''list attached files'''
        return self.objectValues('File')

    security.declareProtected(AccessControl.Permissions.view, 'listLog')
    def listLog(self):
        '''return log''' 
        return self.log

    # Schema used by add/edit forms

    security.declareProtected(AccessControl.Permissions.view, 'schema')
    schema = PageTemplateFile('schema.pt', _wwwDir)

    security.declareProtected(permissions.EDIT_BUG,'admin')
    admin = PageTemplateFile('bug_admin.pt', _wwwDir)    

def containerFilter(container):
    return interfaces.IBugRepository.isImplementedBy(container)

def manage_addBug(container, REQUEST):
    """Bug factory:
    Just redirect to the BugRepository constructor
    """
    REQUEST.RESPONSE.redirect(container.absolute_url() + "/manage_addBugForm")

bugConstructors=(manage_addBug,)


Globals.InitializeClass(Bug)
